﻿
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

/*                      Neville interpolatopn                   */
/*                       www.mosismath.com                      */


namespace WindowsFormsApplication1
{

   
    public partial class MainWin : Form
    {
        const int order = 6; // number of reference points
        const int datapoints = 1000; // number of datapoints for the interpolation
        const double deltaX = 1.0; // sample time 0.4

        double[] y_k = new double[order];  // Supporting points
        double[] x_k = new double[order];
        double[] yp = new double[datapoints]; // Data points for graphic
        double[] xp = new double[datapoints];
        List<double[]> NevillePoly = new List<double[]>();
        List<double[]> tempNeville = new List<double[]>();


        public MainWin()
        {
            int i;
            DataGridViewCell cell;
           
            InitializeComponent();

            dGPolynom.ColumnCount = order;
            dGPolynom.RowCount = 2;

            // init some sample y values
            y_k[0] = 0.5;
            y_k[1] = 11.7;
            y_k[2] = 14.8;
            y_k[3] = 4.0;
            y_k[4] = 2.2;
            y_k[5] = 0.2;

        
            // timestamp to y value
            for (i = 0; i < order; i++)
            {
                x_k[i] = deltaX * (double)(i+1);
            }

            cell = dGPolynom.Rows[0].HeaderCell;
            cell.Value = "y_k";
            cell = dGPolynom.Rows[1].HeaderCell;
            cell.Value = "x_k";
            for (i = 0; i < order; i++)
            {
                dGPolynom.Columns[i].Name = "a" + i.ToString();
                cell = dGPolynom[i, 0];
                cell.Value = Convert.ToString(y_k[i]);
                cell = dGPolynom[i, 1];
                cell.Value = Convert.ToString(x_k[i]);
            }
        }

        double[] CalcOnePoly(double x_start, double x_end, double[] p_l, double[] p_r, double range)
        {
            double[] tempP = { -1, x_end};
            double[] tempPoly1 = Poly.Mult(p_l, tempP);
            tempP[0] = 1;
            tempP[1] = -x_start;
            double[] tempPoly2 = Poly.Mult(p_r, tempP);
            return Poly.Divide(Poly.Add(tempPoly1, tempPoly2), range);
        }

        void CalcNeville() 
        {
            int i, j;
            j = 0;
            while(NevillePoly.Count > 1)
            {
                j++;
                tempNeville.Clear();
                for (i = 0; i < NevillePoly.Count - 1; i++)
                {
                    double[] tempP1 = NevillePoly[i];
                    double[] tempP2 = NevillePoly[i + 1];
                    tempNeville.Add(CalcOnePoly(x_k[i], x_k[i + j], tempP1, tempP2, j));
                }
                NevillePoly.Clear();
                NevillePoly.AddRange(tempNeville);
            }
        }

        double Interpolate(double xp)
        {  
            double yp;
            double[] p = NevillePoly[0];
            int i, k;

            yp = 0.0;
            for (i = 0; i < p.Length; i++)
            {
                yp += Math.Pow(xp, p.Length - i - 1) * p[i];
            }  
            return yp;
        }


        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            int offset = 20;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime);
                        p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - offset - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - offset - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }

                        g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime);
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    else
                        p1.Y = (pGraph.Height) - offset - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();
        }
    

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i;
            DataGridViewCell cell;

            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    cell = dGPolynom[i, 0];
                    if (cell.ToString() != "")
                    {
                        y_k[i] = Convert.ToDouble(cell.Value);
                        cell = dGPolynom[i, 1];
                        x_k[i] = Convert.ToDouble(cell.Value);
                    }
                    else
                    {
                        y_k[i] = 0.0;
                        x_k[i] = 0.0;
                    }
                }
            }

            NevillePoly.Clear();
            for (i = 0; i < order; i++)
            {
                double[] tempP = { y_k[i] };
                NevillePoly.Add(tempP);
            }
            CalcNeville(); 
            for (i = 0; i < datapoints; i++)
            {
                yp[i] = Interpolate((double)(i) * x_k[order - 1] / (double)(datapoints));
                xp[i] = (double)(i) * x_k[order - 1] / (double)(datapoints);
            }
            DrawGraph(datapoints, x_k[0], xp[xp.Length-1], true);
        }


        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, x_k[0], xp[xp.Length - 1], false);
        }  
    }
}
